/* $Id: vram.c,v 1.1.1.1 1999/01/30 06:09:06 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************
 */

#include "vram.h"
#include "error.h"
#include "rom.h"

static vram_format_t *vram_root;

void vram_add_format (vram_format_t *m)
{
	m->next = vram_root;
	vram_root = m;
}

vram_image_t *vram_new_image (void)
{
	vram_image_t *i = calloc (1,sizeof(vram_image_t));
	
	if (i==0) errorf("malloc(%d) error\n",sizeof(vram_image_t));
	
	return i;
}

void vram_delete_image (vram_image_t *img)
{
	if (img)
	{
		if (img->pixel) free (img->pixel);
		free (img);
	}
}

vram_format_t *vram_detect (char *filename)
{
	vram_format_t *vram = vram_root;
	
	verbosef (VERBOSE_CHECK,"autodetecting file format:\n");
	
	do {
		verbosef (VERBOSE_CHECK,"%s: ",vram->name);
		if (vram->detect(filename)) break;
		verbosef (VERBOSE_CHECK,"NO\n");
		vram = vram->next;
	} while (vram != NULL);

	if (vram) verbosef (VERBOSE_CHECK,"YES\n");

	return vram;
}

void vram_list (void)
{
	vram_format_t *vram = vram_root;
		
	do {
		verbosef (VERBOSE_ALL,"\"%s\" ",vram->name);
		vram = vram->next;
	} while (vram != NULL);

	verbosef (VERBOSE_ALL,"\n");
}

void vram_operation (u_short x,u_short y,u_short w,u_short h,u_short f,char *filename,bool upload)
{
	vram_image_t *img = vram_new_image();
	vram_format_t *vram = NULL;

	img->x = x;
	img->y = y;
	img->w = w;
	img->h = h;
	img->f = f;

	if (filename==NULL)
	{
		filename = "image.tim";
		warnf ("filename missing; using \"%s\"\n",filename);
	}

	if (upload)
	{
		vram = vram_detect (filename);
		if (vram)
		{
			vram->read (filename,img);
			
			rom->send_vram (img);
		}
		else warnf ("image file format not recognized.\n");
	}
	else
	{
		switch (img->f)
		{
			case 15:
			case 16:
				img->f = 0;
				break;
			case 24:
				img->f = 1;
				break;
			default:
				warnf ("illegal bpp=%d, using default\n",img->f);
			case 0xffff:
				img->f = 0xffff;
				break;
		}

		rom->recv_vram (img);

		if (img->pixel != NULL)
		{
			vram = vram_detect (filename);
			if (vram) vram->write (filename,img);
			else warnf ("image file format not recognized.\n");
		}
		else warnf ("malloc failed\n");
	}

	vram_delete_image(img);
}
